@@ -11,12 +11,13 @@ from typing import Tuple
 from typing import Optional
 from torch.nn import Module
 from torch.nn import ModuleList
+from cftool.misc import filter_kw
 from cftool.misc import safe_execute
 
 from .poolers import BertPooler
 from .poolers import SequencePooler
-from .feedforward import FFN
-from .feedforward import FeedForward
+from .channel_mixers import ChannelMixerBase
+from .channel_mixers import FeedForward
 from .token_mixers import TokenMixerBase
 from ..norms import NormFactory
 from ..utils import zero_module
@@ -32,9 +33,11 @@ from ....misc.toolkit import gradient_checkpoint
 class MixingBlock(Module):
     def __init__(
         self,
+        layer_idx: int,
+        num_layers: int,
         num_tokens: int,
+        in_dim: int,
         latent_dim: int,
-        feedforward_dim: int,
         *,
         norm_position: str = "pre_norm",
         token_mixing_type: str,
@@ -50,7 +53,7 @@ class MixingBlock(Module):
     ):
         def _make_norm() -> nn.Module:
             factory = NormFactory(norm_type)
-            return factory.make(latent_dim, **(norm_kwargs or {}))
+            return factory.make(in_dim, **(norm_kwargs or {}))
 
         super().__init__()
         if norm_position not in {"pre_norm", "post_norm"}:
@@ -65,7 +68,10 @@ class MixingBlock(Module):
             token_mixing_config = {}
         token_mixing_config.update(
             {
+                "layer_idx": layer_idx,
+                "num_layers": num_layers,
                 "num_tokens": num_tokens,
+                "in_dim": in_dim,
                 "latent_dim": latent_dim,
                 "dropout": dropout,
             }
@@ -81,14 +87,17 @@ class MixingBlock(Module):
             channel_mixing_config = {}
         channel_mixing_config.update(
             {
-                "in_dim": latent_dim,
-                "latent_dim": feedforward_dim,
+                "layer_idx": layer_idx,
+                "num_layers": num_layers,
+                "in_dim": in_dim,
+                "latent_dim": latent_dim,
                 "dropout": dropout,
             }
         )
         self.residual_after_norm = residual_after_norm
         self.channel_norm = _make_norm()
-        self.channel_mixing = FFN.make(channel_mixing_type, channel_mixing_config)
+        channel_mixing_base = ChannelMixerBase.get(channel_mixing_type)
+        self.channel_mixing = safe_execute(channel_mixing_base, channel_mixing_config)
 
     def forward(
         self,
@@ -247,7 +256,7 @@ class PositionalEncoding(Module):
 class MixedStackedEncoder(Module):
     def __init__(
         self,
-        dim: int,
+        in_dim: int,
         num_tokens: int,
         *,
         token_mixing_type: str,
@@ -264,7 +273,8 @@ class MixedStackedEncoder(Module):
         embedding_norm: Optional[nn.Module] = None,
         embedding_dropout: Optional[float] = None,
         residual_after_norm: bool = False,
-        feedforward_dim_ratio: float = 1.0,
+        latent_dim: Optional[int] = None,
+        latent_dim_ratio: float = 1.0,
         use_head_token: bool = False,
         head_pooler: Optional[str] = "mean",
         use_positional_encoding: bool = False,
@@ -284,7 +294,7 @@ class MixedStackedEncoder(Module):
             num_head_tokens = 1
             if aux_heads is not None:
                 num_head_tokens += len(aux_heads)
-            self.head_token = nn.Parameter(torch.zeros(1, num_head_tokens, dim))
+            self.head_token = nn.Parameter(torch.zeros(1, num_head_tokens, in_dim))
         self.num_heads = num_head_tokens
         # positional encoding
         num_tokens += num_head_tokens
@@ -296,7 +306,7 @@ class MixedStackedEncoder(Module):
                 )
             is_vision_positional_encoding = False
         self.pos_encoding = PositionalEncoding(
-            dim,
+            in_dim,
             num_tokens,
             positional_encoding_dropout,
             num_head_tokens=num_head_tokens,
@@ -311,13 +321,16 @@ class MixedStackedEncoder(Module):
         # core
         if dpr_list is None:
             dpr_list = [x.item() for x in torch.linspace(0, drop_path_rate, num_layers)]
-        feedforward_dim = int(round(dim * feedforward_dim_ratio))
+        if latent_dim is None:
+            latent_dim = int(round(in_dim * latent_dim_ratio))
         self.mixing_blocks = ModuleList(
             [
                 MixingBlock(
+                    i,
+                    num_layers,
                     num_tokens,
-                    dim,
-                    feedforward_dim,
+                    in_dim,
+                    latent_dim,
                     norm_position=norm_position,
                     token_mixing_type=token_mixing_type,
                     token_mixing_config=token_mixing_config,
@@ -329,7 +342,7 @@ class MixedStackedEncoder(Module):
                     norm_kwargs=norm_kwargs,
                     residual_after_norm=residual_after_norm,
                 )
-                for drop_path in dpr_list
+                for i, drop_path in enumerate(dpr_list)
             ]
         )
         # head
@@ -342,7 +355,7 @@ class MixedStackedEncoder(Module):
             if head_pooler is None:
                 head = nn.Identity()
             elif head_pooler == "sequence":
-                head = SequencePooler(dim, aux_heads)
+                head = SequencePooler(in_dim, aux_heads)
             else:
                 if aux_heads is not None:
                     raise ValueError(
@@ -350,7 +363,7 @@ class MixedStackedEncoder(Module):
                         f"when `aux_heads` ({aux_heads}) is provided"
                     )
                 if head_pooler == "bert":
-                    head = BertPooler(dim)
+                    head = BertPooler(in_dim)
                 elif head_pooler == "mean":
                     head = Lambda(lambda x: x.mean(1), name="global_average")
                 else:
@@ -363,12 +376,12 @@ class MixedStackedEncoder(Module):
             self.head_norm = None
             self.head = head
         elif norm_after_head:
-            self.head_norm = NormFactory(norm_type).make(dim, **(norm_kwargs or {}))
+            self.head_norm = NormFactory(norm_type).make(in_dim, **(norm_kwargs or {}))
             self.head = head
         else:
             self.head_norm = None
             self.head = PreNorm(
-                dim,
+                in_dim,
                 module=head,
                 norm_type=norm_type,
                 norm_kwargs=norm_kwargs,
@@ -403,7 +416,8 @@ class MixedStackedEncoder(Module):
             t += 1
         if determinate:
             net = net.view(-1, *map(int, [t, d]))
-        net = self.pos_encoding(net, **kwargs)
+        kw = filter_kw(self.pos_encoding.forward, kwargs)
+        net = self.pos_encoding(net, **kw)
         if self.embedding_norm is not None:
             net = self.embedding_norm(net)
         if self.embedding_dropout is not None:
