@@ -22,7 +22,6 @@ from collections import OrderedDict
 from cftool.misc import random_hash
 from cftool.misc import safe_execute
 from cftool.misc import print_warning
-from cftool.misc import check_requires
 from cftool.misc import shallow_copy_dict
 from cftool.misc import prepare_workplace_from
 from cftool.misc import truncate_string_to_length
@@ -59,7 +58,7 @@ from .constants import CHECKPOINTS_FOLDER
 from .constants import BATCH_INDICES_KEY
 from .misc.toolkit import get_ddp_info
 from .misc.toolkit import _get_environ_workplace
-from .misc.internal_.trainer import make_trainer
+from .implementations.trainer import make_trainer
 
 
 pipeline_dict: Dict[str, Type["IPipeline"]] = {}
@@ -119,12 +118,12 @@ def _generate_model_soup_checkpoint(
                     current_states[k] = (current_v * n + v) / (n + 1)
             m.model.load_state_dict(current_states)
             kw = dict(
+                loader=configs.loader,
                 portion=configs.valid_portion,
                 metrics=metrics,
+                use_loader_cache=False,
             )
-            if check_requires(m.inference.get_outputs, "use_loader_cache"):
-                kw["use_loader_cache"] = False
-            res = m.inference.get_outputs(configs.loader, **kw)  # type: ignore
+            res = safe_execute(m.inference.get_outputs, kw)
             score = res.metric_outputs.final_score  # type: ignore
             if score < best_score:
                 current_states = states_backup
@@ -299,8 +298,12 @@ class IModifier(WithRegister["IModifier"], IDLPipeline):
                 f"{', '.join(missing_requirements)}"
             )
 
-    @staticmethod
-    def _report_messages(title: str, messages: Dict[str, Any]) -> None:
+    def _report_messages(
+        self,
+        title: str,
+        messages: Dict[str, Any],
+        report_folder: Optional[str],
+    ) -> None:
         def _stringify_item(item: Tuple[str, Any], prefix: Optional[str] = None) -> str:
             key, value = item
             if prefix is not None:
@@ -314,26 +317,29 @@ class IModifier(WithRegister["IModifier"], IDLPipeline):
 
         span = 64
         length = 2 * span
-        print(
-            "\n".join(
-                [
-                    "=" * length,
-                    f"{title:^{length}s}",
-                    "-" * length,
-                    "\n".join(map(_stringify_item, messages.items())),
-                    "-" * length,
-                ]
-            )
+        msg = "\n".join(
+            [
+                "=" * length,
+                f"{title:^{length}s}",
+                "-" * length,
+                "\n".join(map(_stringify_item, messages.items())),
+                "-" * length,
+            ]
         )
+        print(msg)
+        if report_folder is not None:
+            with open(os.path.join(report_folder, self.report_file), "a") as f:
+                f.write(msg + "\n")
 
-    def _report_defaults(self) -> None:
+    def _report_defaults(self, report_folder: Optional[str] = None) -> None:
         self._report_messages(
             "Internal Default Configurations Used by `carefree-learn`",
             self._defaults,
+            report_folder,
         )
 
-    def _report_configs(self) -> None:
-        self._report_messages("External Configurations", self.config)
+    def _report_configs(self, report_folder: Optional[str] = None) -> None:
+        self._report_messages("External Configurations", self.config, report_folder)
 
     # build steps
 
@@ -427,14 +433,17 @@ class IModifier(WithRegister["IModifier"], IDLPipeline):
         *,
         build_trainer: bool = True,
         report: Optional[bool] = None,
+        report_folder: Optional[str] = None,
     ) -> None:
         if self.built:
             if build_trainer and self.trainer is None:
                 self.need_build_trainer = True
                 self.record_num_samples()
                 self.prepare_workplace()
-                self.build_trainer()
-                self.report()
+                workplace = self.build_trainer()
+                if report_folder == self.auto_report_key:
+                    report_folder = workplace
+                self.report(report_folder)
             return None
         self.data_info = shallow_copy_dict(data_info)
         self.need_build_trainer = build_trainer
@@ -447,7 +456,9 @@ class IModifier(WithRegister["IModifier"], IDLPipeline):
         if not build_trainer:
             self.trainer = None  # type: ignore
         else:
-            self.build_trainer()
+            workplace = self.build_trainer()
+            if report_folder == self.auto_report_key:
+                report_folder = workplace
         self._sanity_check()
         if report is None:
             if self.trainer is None:
@@ -457,19 +468,20 @@ class IModifier(WithRegister["IModifier"], IDLPipeline):
                 in_distributed = self.trainer.tqdm_settings.in_distributed
                 report = is_rank_0 and not in_distributed
         if report:
-            self.report()
+            self.report(report_folder)
         self.built = True
 
-    def build_trainer(self) -> None:
+    def build_trainer(self) -> str:
         trainer_config = self.trainer_config.copy()
         if isinstance(self.model, ModelWithCustomSteps):
             self.model.permute_trainer_config(trainer_config)
         self.trainer = make_trainer(trainer_config)
+        return self.trainer.workplace
 
-    def report(self) -> None:
+    def report(self, report_folder: Optional[str] = None) -> None:
         if self.is_rank_0:
-            self._report_defaults()
-            self._report_configs()
+            self._report_defaults(report_folder)
+            self._report_configs(report_folder)
 
     # load steps
 
@@ -586,6 +598,8 @@ class DLPipeline(IPipeline, IDLPipeline):
     inference_base = IInference
     config_base: Type[Config] = DLConfig
 
+    report_file = "report.txt"
+    auto_report_key = "^_^auto_report^_^"
     pipeline_key = "pipeline"
     pipeline_file = "pipeline.txt"
     config_name = "config"
@@ -676,9 +690,15 @@ class DLPipeline(IPipeline, IDLPipeline):
         *,
         build_trainer: bool = True,
         report: Optional[bool] = None,
+        report_folder: Optional[str] = None,
     ) -> None:
         modifier = self._make_modifier()
-        modifier.build(data_info, build_trainer=build_trainer, report=report)
+        modifier.build(
+            data_info,
+            build_trainer=build_trainer,
+            report=report,
+            report_folder=report_folder,
+        )
 
     def fit(  # type: ignore
         self,
@@ -691,7 +711,7 @@ class DLPipeline(IPipeline, IDLPipeline):
         if cuda is not None:
             cuda = str(cuda)
         self.data = data
-        self.build(data.info)
+        self.build(data.info, report_folder=self.auto_report_key)
         self.trainer.fit(
             data,
             self.loss,
@@ -819,6 +839,7 @@ class DLPipeline(IPipeline, IDLPipeline):
         compress: bool = True,
         build_trainer: bool = False,
         report: Optional[bool] = None,
+        report_folder: Optional[str] = None,
         states_callback: states_callback_type = None,
         pre_callback: Optional[Callable[[Dict[str, Any]], None]] = None,
         post_callback: Optional[Callable[["DLPipeline", Dict[str, Any]], None]] = None,
@@ -852,7 +873,12 @@ class DLPipeline(IPipeline, IDLPipeline):
                     )
                     data_info = {}
                 modifier = m._make_modifier()
-                modifier.build(data_info, build_trainer=build_trainer, report=report)
+                modifier.build(
+                    data_info,
+                    build_trainer=build_trainer,
+                    report=report,
+                    report_folder=report_folder,
+                )
                 m.model.to(m.device)
                 # restore checkpoint
                 states = modifier.load_states_from(export_folder)
