@@ -34,6 +34,7 @@ from cftool.misc import context_error_handler
 from cftool.misc import get_num_positional_args
 from cftool.misc import DataClassBase
 from cftool.misc import WithRegister
+from cftool.array import to_torch
 from cftool.array import to_numpy
 from cftool.array import to_device
 from cftool.types import np_dict_type
@@ -82,8 +83,8 @@ multi_prefix_mapping: Dict[str, Type["MultiLoss"]] = {}
 metric_dict: Dict[str, Type["_IMetric"]] = {}
 callback_dict: Dict[str, Type["TrainerCallback"]] = {}
 
-LossType = TypeVar("LossType", bound="ILoss", covariant=True)
-ConfigType = TypeVar("ConfigType")
+TLoss = TypeVar("TLoss", bound="ILoss", covariant=True)
+TDataModule = TypeVar("TDataModule")
 
 
 # data
@@ -103,6 +104,7 @@ class IDataset(WithRegister["IDataset"], metaclass=ABCMeta):
 class IDataLoader(WithRegister["IDataLoader"], metaclass=ABCMeta):
     d = loader_dict
     data: IDataset
+    use_numpy: bool
     batch_size: int
 
     def __init__(self, *, sample_weights: Optional[np.ndarray] = None):
@@ -156,7 +158,7 @@ class IDataModule(ABC):
         pass
 
     @abstractmethod
-    def prepare(self, sample_weights: sample_weights_type) -> None:
+    def prepare(self: TDataModule, sample_weights: sample_weights_type) -> TDataModule:
         pass
 
     @abstractmethod
@@ -598,7 +600,7 @@ class MonitorResults(NamedTuple):
 # loss
 
 
-class ILoss(nn.Module, WithRegister[LossType], metaclass=ABCMeta):
+class ILoss(nn.Module, WithRegister[TLoss], metaclass=ABCMeta):
     d = loss_dict
     placeholder_key = "[PLACEHOLDER]"
 
@@ -789,6 +791,13 @@ class PlaceholderLoss(ILoss):
 # inference
 
 
+def np_batch_to_tensor(np_batch: np_dict_type) -> tensor_dict_type:
+    return {
+        batch_key: None if batch_array is None else to_torch(batch_array)
+        for batch_key, batch_array in np_batch.items()
+    }
+
+
 class IInference:
     def __init__(
         self,
@@ -823,33 +832,29 @@ class IInference:
             metric_outputs_list: List[MetricsOutputs] = []
             loss_items: Dict[str, List[float]] = {}
             labels: Dict[str, List[np.ndarray]] = {}
+            loader.use_numpy = True
             iterator = enumerate(loader)
             if use_tqdm:
                 iterator = tqdm(iterator, "inference", len(loader))
             requires_all_outputs = return_outputs
             if metrics is not None and metrics.requires_all:
                 requires_all_outputs = True
-            for i, batch in iterator:
+            for i, np_batch in iterator:
                 if i / len(loader) >= portion:
                     break
-                np_batch = {
-                    batch_key: None if batch_tensor is None else to_numpy(batch_tensor)
-                    for batch_key, batch_tensor in batch.items()
-                }
-                if self.model is not None:
-                    batch = to_device(batch, self.model.device)
+                batch = None
+                if self.model is not None or loss is not None:
+                    batch = np_batch_to_tensor(np_batch)
+                    if self.model is not None:
+                        batch = to_device(batch, self.model.device)
                 if self.onnx is not None:
-                    local_outputs = self.onnx.predict(np_batch)
+                    local_outputs = np_batch_to_tensor(self.onnx.predict(np_batch))
                 else:
                     assert self.model is not None
                     with eval_context(self.model, use_grad=use_grad):
                         assert not self.model.training
-                        local_outputs = self.model(
-                            i,
-                            batch,
-                            state,
-                            **shallow_copy_dict(kwargs),
-                        )
+                        kw = filter_kw(self.model.forward, shallow_copy_dict(kwargs))
+                        local_outputs = self.model(i, batch, state, **kw)
                 # gather outputs
                 requires_metrics = metrics is not None and not metrics.requires_all
                 requires_np = requires_metrics or requires_all_outputs
@@ -877,7 +882,7 @@ class IInference:
                     else:
                         results.setdefault(k, []).append(v_np)  # type: ignore
                 if requires_np:
-                    for k, v in batch.items():
+                    for k, v in np_batch.items():
                         if (
                             k == INPUT_KEY
                             or k == ORIGINAL_LABEL_KEY
@@ -888,8 +893,6 @@ class IInference:
                             continue
                         if k != LABEL_KEY and len(v.shape) > 2:
                             continue
-                        if not isinstance(v, np.ndarray):
-                            v = to_numpy(v)
                         labels.setdefault(k, []).append(v)
                 # metrics
                 if requires_metrics:
@@ -939,6 +942,7 @@ class IInference:
                     {k: sum(vl) / len(vl) for k, vl in metric_values.items()},
                 )
 
+            loader.use_numpy = False
             target_labels = labels.get(LABEL_KEY, [])
             return InferenceOutputs(
                 final_results,
